/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.autoload.AutoLoaderService;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task creates state holder entry for component, service assembly or
 * shared lib <br/> Inputs : {@link Jbi}, archive {@link URI}
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public abstract class AbstractCreateStateHolderTask extends AbstractFileManipulationTask {

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemStateService systemStateSrv;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public AbstractCreateStateHolderTask(SystemStateService systemStateSrv,
            RepositoryService repositorySrv, AutoLoaderService autoLoaderSrv, LoggingUtil log) {
        super(autoLoaderSrv);
        this.systemStateSrv = systemStateSrv;
        this.repositorySrv = repositorySrv;
        this.log = log;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);

        File file = getArchiveFile(archiveURI);
        File installedArchive = getInstalledArchive(file);

        createEntityStateHolder(descriptor, installedArchive);

    }

    protected abstract void createEntityStateHolder(Jbi descriptor, File installedArchive)
            throws Exception;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        try {
            deleteEntityStateHolder(descriptor);
        } catch (Exception e) {
            String msg = "Failed to revert a CreateStateHolderTask";
            log.error(msg, e);
        }

    }

    protected abstract void deleteEntityStateHolder(Jbi descriptor) throws Exception;

}
