/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.sl;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import org.ow2.petals.jbi.management.autoload.AutoLoaderService;
import org.ow2.petals.jbi.management.task.AbstractFileManipulationTask;
import org.ow2.petals.jbi.management.task.installation.InstallationContextConstants;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.platform.systemstate.SharedLibraryState;

/**
 * This task moves a shared lib archive from the installed Petals directory to
 * the uninstalled directory <br/> Inputs : {@link SharedLibraryState}
 * 
 * @author ofabre
 * 
 */
public class DeleteArchiveFromInstalledDirTask extends AbstractFileManipulationTask {

    private PackageHelper packageHandler;

    public DeleteArchiveFromInstalledDirTask(AutoLoaderService autoLoaderSrv,
            PackageHelper packageHandler) {
        super(autoLoaderSrv);
        this.packageHandler = packageHandler;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        SharedLibraryState slState = (SharedLibraryState) context
                .get(InstallationContextConstants.SHARED_LIB_STATE);

        File installedArchive = null;
        if (slState != null) {
            URI archiveURI = packageHandler.getPackageURI(slState.getArchiveURL(), false);
            installedArchive = new File(archiveURI);

        }

        // Copy archive only if the uninstalled directory exists, in some
        // distribution there is no install/installed/uninstalled directories,
        // only one directory.
        if (installedArchive.exists()) {
            File uninstalledArchive = getUninstalledArchive(installedArchive);
            if (uninstalledArchive != null) {
                installedArchive.renameTo(uninstalledArchive);
            }
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // Nothing to do
    }

}
