/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing.module;

import javax.jbi.messaging.MessageExchange;

import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.routing.AddressResolver;
import org.ow2.petals.jbi.messaging.routing.RoutingException;

/**
 * Router module to resolve address.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 2.2
 * 
 */
public class AddressResolverModule extends RouterModule {

    protected AddressResolver addressResolver;

    /**
     * Creates a new instance of AddressResolverModule
     */
    public AddressResolverModule() {
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.messaging.routing.module.RouterModule#doInit()
     */
    @Override
    protected void doInit() throws Exception {
        this.addressResolver = new AddressResolver(this.configurationService, this.topologyService,
                this.endpointService);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.messaging.routing.module.RouterModule#doStart()
     */
    @Override
    protected void doStart() throws Exception {
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.messaging.routing.module.RouterModule#doStop()
     */
    @Override
    protected void doStop() throws Exception {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#send(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, long)
     */
    public void send(ComponentContextImpl source, MessageExchangeImpl exchange, long timeOut)
            throws RoutingException {

        if (MessageExchange.Role.CONSUMER.equals(exchange.getRole())) {
            this.resolveAddress(source, exchange);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#sendSync(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, long)
     */
    public MessageExchangeImpl sendSync(ComponentContextImpl source, MessageExchangeImpl exchange,
            long timeOut) throws RoutingException {

        if (MessageExchange.Role.CONSUMER.equals(exchange.getRole())) {
            this.resolveAddress(source, exchange);
        }

        return exchange;
    }

    /**
     * Resolving the address is finding the right endpoint
     * 
     * @param source
     * @param exchange
     * @throws RoutingException
     */
    protected void resolveAddress(final ComponentContextImpl source,
            final MessageExchangeImpl exchange) throws RoutingException {
        this.addressResolver.resolveAddress(source, exchange);
    }
}
