/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.kernel.configuration;

import org.ow2.petals.topology.TopologyBuilder;
import org.ow2.petals.topology.TopologyException;
import org.ow2.petals.topology.generated.Topology;
import org.ow2.petals.communication.topology.TopologyService;

/**
 * The interface of the Configuration service. The service handles the
 * configuration of the local PEtALS container.<br>
 * At its starting, it retrieves the configuration from the
 * {@link TopologyBuilder} parser and the server.properties file , it performs a
 * first level validation of the consistency, then it fill the local
 * {@link ContainerInformation}.<br>
 * It provides provides a method to validate an new container against the
 * topology. Note that for the dynamic distributed mode, some validation is
 * performed by the {@link TopologyService}.
 *
 * @author Roland NAUDIN - EBMWebsourcing
 * @since Petals 2.0
 */
public interface ConfigurationService {

    /**
     * Name of the topology file.
     */
    static final String TOPOLOGY_FILE = "/topology.xml";

    /**
     * Name of the server properties file.
     */
    static final String SERVER_PROPS_FILE = "/server.properties";
    
    static final String JAAS_PROPS_FILE = "/jaas.properties";

    /**
     * Properties keys
     */

    static final String PROPERTY_CONTAINER_NAME = "petals.container.name";

    static final String PROPERTY_REPOSITORY_PATH = "petals.repository.path";

    static final String PROPERTY_EXCHANGE_VALIDATION = "petals.exchange.validation";

    static final String PROPERTY_TRANSPORT_QOS = "petals.transport.qos";

    static final String PROPERTY_ROUTING_STRATEGY = "petals.routing.strategy";

    static final String PROPERTY_TASK_TIMEOUT = "petals.task.timeout";
    
    static final String PROPERTY_CLASSLOADER_ISOLATED = "petals.classloaders.isolated";
    
    static final String PROPERTY_USER_CONFIG_PREFIX = "user";

    // static final String SSL_IDENTITYKEYSTORE = "server.identitykeystore";
    //
    // static final String SSL_IDENTITYKEYSTORE_PWD =
    // "server.identitykeystore.password";
    //
    // static final String SSL_TRUSTEDKEYSTORE = "server.trustedkeystore";
    //
    // static final String SSL_TRUSTEDKEYSTORE_PWD =
    // "server.trustedkeystore.password";

    /**
     * Get the local container configuration.
     */
    ContainerConfiguration getContainerConfiguration();

    /**
     * Get the sub domain configuration.
     */
    SubDomainConfiguration getSubDomainConfiguration();

    /**
     * Get the domain configuration.
     */
    DomainConfiguration getDomainConfiguration();

    /**
     * Get the Static topology
     *
     * @return
     */
    Topology getStaticTopology() throws ConfigurationException;

    /**
     * Set the running state of the container.
     *
     * @param state
     *            {@code ContainerConfiguration.ContainerState}
     */
    void setContainerState(ContainerConfiguration.ContainerState state);

    /**
     * Verify the given {@code containerConfiguration} against the topology
     * configuration.
     *
     * @param newContainerConfiguration
     *            The new container configuration
     * @param newSubDomainConfiguration
     *            The new sub-domain configuration
     * @param newDomainConfiguration
     *            The new domain configuration
     * @throws TopologyException
     *             Inconsistency between the topology configuration and the new
     *             container configuration
     */
    void verifyContainerTopology(
        final ContainerConfiguration newContainerConfiguration,
        final SubDomainConfiguration newSubDomainConfiguration,
        final DomainConfiguration newDomainConfiguration) throws TopologyException;
}
